/**
 * @file
 * Vue app for cart.
 */

(function () {
  "use strict";

  Vue.component("cart-field", {
    props: {
      pid: 0,
      variations: false,
      attributes: false,
      fullstate: 0,
    },
    data() {
      return {
        inCart: false,
        ready: false,
        loading: false,
        variationId: false,
        oldPrice: false,
        price: false,
        attributeCount: false,
        activeIds: [],
        disableIds: [],
      };
    },
    methods: {
      addToCart(variation_id) {
        if (this.loading || !this.variationId) {
          return false;
        }
        this.loading = true;
        axios.post("/cart/add-item", { vid: variation_id }).then(
          (response) => {
            this.inCart = true;
            this.loading = false;
            this.$root.$emit("update_cart", response.data);
          },
          (response) => {
            this.loading = false;
          }
        );
      },
      choiceOfAttribute(id) {
        this.inCart = false;
        let variationPerf = [];
        this.variationId = 0;
        if (this.activeIds.includes(id)) {
          this.activeIds.splice(this.activeIds.indexOf(id), 1);
        }
        else {
          if (this.attributeCount > 1) {
            this.activeIds.push(id);
          }
          else {
            this.activeIds.splice(0, 1);
            this.activeIds.push(id);
          }

        }
        switch (this.activeIds.length) {
          case 0:
            this.disableIds = [];
            this.variationId = 0;
            break;
          case 1:
            this.disableIds = [];
            if (this.attributeCount > 1) {
              for (var attr in this.attributes) {
                this.attributes[attr].forEach(v => {
                  if (!this.activeIds.includes(v.id)) {
                    this.disableIds.push(v.id);
                  }
                });
              }
              if (this.activeIds.length != this.attributeCount) {
                this.variations.forEach(e => {
                  let b = false;
                  e.attributes.forEach(a => {
                    if (this.activeIds.includes(a.id)) {
                      b = true;
                    }
                  });
                  if (b) {
                    e.attributes.forEach(a => {
                      if (!this.activeIds.includes(a.id) && this.disableIds.includes(a.id)) {
                        this.disableIds.splice(this.disableIds.indexOf(a.id), 1);
                      }
                    });
                  }
                });
              }
            }
            break;
          default:
            if (this.activeIds.length != this.attributeCount) {
              this.variations.forEach(e => {
                let b = false;
                let q = 0
                e.attributes.forEach(a => {
                  if (this.activeIds.includes(a.id)) {
                    b = true;
                    q++;
                  }
                });
                if (b) {
                  e.attributes.forEach(a => {
                    if (this.activeIds.length != q) {
                      b = false;
                      this.variations.forEach(e1 => {
                        let q1 = 0
                        let x = [];
                        e1.attributes.forEach(a1 => {
                          x.push(a1.id);
                          if (this.activeIds.includes(a1.id)) {
                            q1++;
                          }
                        });
                        if (this.activeIds.length == q1) {
                          variationPerf = x;
                        }
                      });
                      if (!this.activeIds.includes(a.id) && !this.disableIds.includes(a.id)) {
                          if (!variationPerf.includes(a.id)) {
                            this.disableIds.push(a.id);
                          }
                      }
                    }
                  });
                }
              });
            }
            break;
        }
        if (this.activeIds.length == this.attributeCount) {
          this.variations.forEach(e => {
            let q = 0;
            e.attributes.forEach(a => {
              if (this.activeIds.includes(a.id)) {
                q++;
              }
            });
            if (this.activeIds.length == q) {
              this.variationId = e.variation_id;
              this.oldPrice = e.oldprice_format;
              this.price = e.price_format;
            }
          });
        }
      },
    },
    created() {
      this.ready = true;
      let variationsCount = Object.keys(this.variations).length;
      if (typeof this.variations == 'object') {
        this.oldPrice = this.variations[0].oldprice_format;
        this.price = this.variations[0].price_format;
        if (variationsCount == 1) {
          this.variationId = this.variations[0].variation_id;
        }
      }
      if (typeof this.attributes == 'object') {
        this.attributeCount = 0;
        for (var attr in this.attributes) {
          this.attributeCount++;
          if (variationsCount == 1) {
            this.attributes[attr].forEach(v => {
              this.activeIds.push(v.id);
            });
          }
        }
        if (this.attributeCount < 1 ) {
          this.variationId = this.variations[0].variation_id;
        }
      }
    },
  });

  Vue.component("cart", {
    data() {
      return {
        cart: {
          items: {},
          quantity: 0,
          price: 0,
          total: 0,
          subtotal: 0,
        },
        ready: false,
      };
    },
    methods: {
      initCart() {
        let loadPath = "/api/cart-load";
        let path = drupalSettings.path.currentPath;
        let arg = path.split("/");
        if (arg.length == 3) {
          if (arg[0] == "checkout" && parseInt(arg[1]) > 0) {
            loadPath += "/" + arg[1];
          }
        }
        axios.get(loadPath).then((response) => {
          this.cart = response.data;
          if (this.cart.quantity > 0) {
            this.ready = true;
          }
          this.$root.$emit("update_cart", this.cart);
        });
      },
      changeQuantity(vid, quantity) {
        quantity = !quantity || quantity < 1 ? 1 : quantity;
        let stock = this.cart.items[vid].stock;
        if (quantity <= stock || stock == 0) {
          axios
            .post("/cart/set-product-quantity", {
              vid: vid,
              quantity: quantity,
            })
            .then((response) => {
              this.cart = response.data;
              this.initCart();
            });
        }
      },
      removeItem(vid) {
        axios.post("/cart/remove-cart-item", { vid: vid }).then((response) => {
          this.ready = false;
          this.initCart();
        });
      },
      numberFormat(number, decimals = 2, dec_point = ".", thousands_sep = " ") {
        let s_number =
          Math.abs(parseInt((number = (+number || 0).toFixed(decimals)))) + "";
        let len = s_number.length;
        let tchunk = len > 3 ? len % 3 : 0;
        let ch_first = tchunk ? s_number.substr(0, tchunk) + thousands_sep : "";
        let ch_rest = s_number
          .substr(tchunk)
          .replace(/(\d\d\d)(?=\d)/g, "$1" + thousands_sep);
        let ch_last = decimals
          ? dec_point + (Math.abs(number) - s_number).toFixed(decimals).slice(2)
          : "";
        return ch_first + ch_rest + ch_last;
      },
      isEmpty(value) {
        if (!!value && value instanceof Array) {
          return value.length < 1;
        }
        if (!!value && typeof value === "object") {
          for (var key in value) {
            if (hasOwnProperty.call(value, key)) {
              return false;
            }
          }
        }
        return !value;
      },
      readQuantity(e, item) {
        let quantity = e.target.innerText;
        quantity = parseInt(quantity.replaceAll(" ", ""));
        if (quantity > 999) {
          quantity = 999;
        } else if (quantity < 1) {
          quantity = 1;
        }
        this.changeQuantity(item.id, quantity);
      },
    },
    created() {
      this.initCart();
    },
  });

  String.prototype.replaceAll = function (search, replacement) {
    return this.split(search).join(replacement);
  };
})();
